/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

package dd.util;

import java.io.Serializable;
import java.util.*;

/** A set of items with its support. */
public class FrequentItemset  implements Comparable<FrequentItemset>, Serializable
{
	/** Serial ID. */
	private static final long serialVersionUID = -5819566846531530506L;
	/** Itemset support. */
	private final int support;
	/** Items. */
	private ArrayList<Integer> items;
	
	/** Constructor. */
	public FrequentItemset(int totItems, int supp) 
	{
		support = supp;
		items = new ArrayList<Integer>();
	}
	/** Parse string format "i-1 i-2 ... i-n (supp)" or "i-1 i-2 ... i-n". */
	public static FrequentItemset parse(int totItems, String specs)
	{
 	    int bracket = specs.indexOf("(");
 	    String ids = bracket < 0 ? specs : specs.substring(0, bracket);
        int support = 0;
  	    if( bracket >= 0 ) {
  	 	    String meta = specs.substring(bracket+1, specs.lastIndexOf(")"));
  	 	    StringTokenizer st = new StringTokenizer(meta, " ");
  	    	support = Integer.parseInt(st.nextToken());
  	    }
 	    StringTokenizer st = new StringTokenizer(ids, " ");
        int nt = st.countTokens();
        FrequentItemset itemset = new FrequentItemset(totItems, support); 
        for(int i=0; i<nt; ++i)
        	itemset.items.add(new Integer(st.nextToken()));
  	    return itemset;
	}
	/** Number of items. */
	public int size() {
		return items.size();
	}
	/** Return items. */
	public ArrayList<Integer> getItems() {
		return items;
	}
	/** Get support. */
	public int getSupport() {
		return support;
	}
	/** String representation. */
	public String toString()
	{
		String res = "";
		for(int i : items)
			res += i + " ";
		return res+"("+support+")";
	}
	@Override
	public int compareTo(FrequentItemset o) {
		int sz = items.size();
		int osz = o.items.size();
		if(sz<osz)
			return -1;
		if(sz>osz)
			return 1;
		Iterator<Integer> it = items.iterator();
		Iterator<Integer> oit = o.items.iterator();
		while(it.hasNext()) {
			Integer v = it.next();
			Integer ov = oit.next();
			if(v<ov)
				return -1;
			if(v>ov)
				return 1;
		}
		return 0;
	}
	public void set(int i) {
		items.add(i);
	}
}
