/*    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2013-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */

 package rst.table;

import java.lang.RuntimeException;
import java.util.Arrays;

/** A column of indexes to a type. */
public class Column  {
	/** Column name. */
	public String name;
	/** Index of values in the column. */
	public int index[];
	
	/** Constructor. */
	public Column(String name, int size) {
		this.name = name;
		if( size <= 0)
			throw new RuntimeException("Column - trying to create a negative size column " + name);
		index = new int[size];
		Arrays.fill(index, -1);
	}
	
	/** Number of distinct values. */
	public int getNDistinct() {
		throw new RuntimeException("Column::getNDistinct - trying to set string for a unknown type on column " + name);
	}

	/** Set value at a pos. */
	public void setString(int pos, String value) {
		throw new RuntimeException("Column::setString - trying to set string for a unknown type on column " + name);
	}
	
	/** Get value at a pos. */
	public String getString(int pos) {
		throw new RuntimeException("Column::getString - trying to set string for a unknown type on column " + name);
	}
	
	/** Get index of a value. */
	public int getIdx(String value) {
		throw new RuntimeException("Column::getIdx - trying to get string for a unknown type on column " + name);
	}
	
	/** Get value of an idx. */
	public String getIdxValues(int idx) {
		throw new RuntimeException("Column::getIdxValues - trying to get string for a unknown type on column " + name);
	}
	
	/** Count how many occurrences of a given index. */
	public int count(int idx) {
		int n = 0;
		for(int v : index)
			if(v == idx)
				++n;
		return n;
	}
	
	/** Get range of indexes. */
	public int getLastIndex() {
		throw new RuntimeException("Column::getLastIndex - for a unknown type on column " + name);
	}

	/** Get Manhattan distance between indexes of two row positions. */
	public double distanceManhattan(int pos1, int pos2) {
		double d = index[pos1] - index[pos2];
		return Math.abs(d);
	}
	
	/** Get Euclidian distance between indexes of two row positions. */
	public double distanceEuclidian(int pos1, int pos2) {
		double d = index[pos1] - index[pos2];
		return d*d;
	}

	/** Get Manhattan distance between indexes of two row positions. */
	public double distanceNormalizedEuclidian(int pos1, int pos2) {
		double d = (index[pos1] - index[pos2])/(double)(getNDistinct()-1);
		return d*d;
	}

	/** Is numeric or discrete. */
	public boolean isNumeric() {
		return false;
	}
}
