/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *    ------------------------------------------------------------------------
 *    Copyright (C) 2004-
 *    Salvatore Ruggieri
 *    Dipartimento di Informatica, Universita' di Pisa
 *    Pisa, Italy
 *    ------------------------------------------------------------------------
 *
 */


package rst.util;

import java.util.ArrayList;
import java.util.Random;

/** Static utility methods dealing with numbers. */
public abstract class NumberUtil {
	
	/** Test method. */
	public static void main(String []argv) {
		int []x = {2, 3, 5, 9};
		int []y = {1, 2, 4, 5, 8, 9, 11};
		System.out.println(merge(x,y));
		System.out.println(mergeUnique(x,y));
	}

	/** Merge ordered arrays. */
	public static ArrayList<Integer> merge(int []x, int []y) {
		ArrayList<Integer> res = new ArrayList<Integer>();
		int posx = 0, posy = 0;
		while(posx < x.length && posy < y.length) {
			int xposx = x[posx];
			int yposy = y[posy];
			if(xposx <= yposy) {
				res.add(xposx);
				posx++;
			}
			else {
				res.add(yposy);
				posy++;
			}
		}
		while(posx < x.length)
			res.add(x[posx++]);
		while(posy < y.length)
			res.add(y[posy++]);
		return res;
	}
	
	/** Merge ordered arrays. */
	public static ArrayList<Integer> mergeUnique(int []x, int []y) {
		ArrayList<Integer> res = new ArrayList<Integer>();
		int posx = 0, posy = 0;
		while(posx < x.length && posy < y.length) {
			int xposx = x[posx];
			int yposy = y[posy];
			if(xposx <= yposy) {
				if(xposx == yposy)
					posy++;
				res.add(xposx);
				posx++;
			}
			else {
				res.add(yposy);
				posy++;
			}
		}
		while(posx < x.length)
			res.add(x[posx++]);
		while(posy < y.length)
			res.add(y[posy++]);
		return res;
	}
	
	/** Shuffle an array randomly. */
	public static void shuffle(int[] a) {
	    int n = a.length;
	    Random random = new Random();
	    random.nextInt();
	    for (int i = 0; i < n; i++) {
	        int change = i + random.nextInt(n - i);
		    int tmp = a[i];
		    a[i] = a[change];
		    a[change] = tmp;
	    }
	}
	
   /** Round a double at a decimal position
     * @param val value to round
     * @param npos number of positions after comma
     * @return rounded value
     */
    public static double round(double val, int npos) {
        int d = 1;
        while (npos > 0) {
            d *= 10;
            --npos;
        }
        return Math.rint(val * d) / d;
    }

    /** Round a double at a the second decimal position
     * after comma.
     * @param val value to round
     * @return rounded value
     */
    public static double round(double val) {
        return round(val, 2);
    }

    /** Compute the mean of a set of integers.
     * @param data an array of integers
     * @param n number of integers in data to be considered
     * @return mean
     */
    public static double avg(int[] data, int n) {
        if (n < 1)
            return Double.NaN;
        long sum = 0;
        for (int i = 0; i < n; ++i)
            sum += data[i];
        return (double) sum / n;
    }

    /** Compute the standard error of a set of integers.
     * @param data an array of integers
     * @param n number of integers in data to be considered
     * @return standard error
     */
    public static double stdErr(int[] data, int n) {
        return stdErr(data, n, avg(data, n));
    }

    /** Compute the standard error of a set of integers.
     * @param data an array of integers
     * @param n number of integers in data to be considered
     * @param mean the mean value
     * @return standard error
     */
    public static double stdErr(int[] data, int n, double mean) {
        if (n < 2)
            return Double.NaN;
        double sum = 0;
        for (int i = 0; i < n; i++) {
            double v = data[i] - mean;
            sum += v * v;
        }
        return Math.sqrt(sum / (n - 1));
    }
}
